
$stretchValue = $args[0]
$scriptPath = $PSScriptRoot
$id = Get-Random

$code = @"
using System;
using System.Collections;
using System.IO;
using System.Net;

namespace SelfieMaskScript
{
    public class Parser$id
    {
        static class Constants
        {
            public const int Timeout = 30000; // Allow 30 seconds for response, per image uploaded
            // public const string WebHostName = "localhost:64951";
            public const string WebHostName = "services.ayespee.net";
            public const string OutputFolder = "output";
            public const string UploadsFolder = "uploads";
            public const string ErrorOccured = "Error Occured: {0}";
            public const string FileTooLarge = "File '{0}' is too large to process. Skipping...";
            public const string ProcessComplete = "Process complete.";
            public const string NoFilesFound = "No files to process.\r\nMake sure your uploads folder contains at least one image.";
        }

        public static int TimeoutCounter { get; set; }

        public static void Main()
        {
            string path = @"$scriptPath";
            string stretchValue = @"$stretchValue";
            string uploadsPath = string.Format(path + @"\{0}", Constants.UploadsFolder);
            string outputPath = string.Format(path + @"\{0}", Constants.OutputFolder);
            string readPath;
            string writePath;
            string[] filters = new string[] { "*.bmp", "*.gif", "*.jpeg", "*.jpg", "*.png" };
            long numBytes;
            double fileLen;
            DirectoryInfo root = new DirectoryInfo(uploadsPath);
            ArrayList images = new ArrayList();
            FileInfo[] files;
            TimeoutCounter = 0;

            try
            {
                foreach (string filter in filters)
                {
                    images.AddRange(root.GetFiles(filter));
                }

                files = (FileInfo[])images.ToArray(typeof(FileInfo));

                if (files != null && files.Length > 0)
                {
                    // Targeting images message
                    Console.WriteLine("Targeting {0} images in: {1}", files.Length.ToString(), uploadsPath);

                    foreach (FileInfo fi in files)
                    {
                        numBytes = fi.Length;
                        fileLen = Convert.ToDouble(numBytes / 1000000);

                        if (fileLen < 4)
                        {
                            if (fi.Name == string.Empty) continue;
                            Console.WriteLine("Processing {0}", fi.Name);

                            readPath = string.Format(uploadsPath + @"\{0}", fi.Name);
                            writePath = string.Format(outputPath + @"\{0}", fi.Name);
                            StretchFile(readPath, writePath, stretchValue);
                        }
                        else
                        {
                            // File too large message
                            Console.WriteLine(Constants.FileTooLarge, fi.Name);
                        }
                    }

                    // Processing complete message
                    Console.WriteLine(Constants.ProcessComplete);
                }
                else
                {
                    // No files to processing message
                    Console.WriteLine(Constants.NoFilesFound);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format(Constants.ErrorOccured, ex.Message), ex);
            }
        }

        private static void StretchFile(string readPath, string writePath, string stretchString)
        {
            try
            {
                // Variables
                WebRequest request;
                byte[] byteArray;
                FileStream fileStream;
                BinaryReader br;
                string serviceUri = string.Format("http://{0}/api/uploading/processfile", Constants.WebHostName);

                // Parse the stretch value
                string stretchValue = stretchString;
                int i;
                if (int.TryParse(stretchValue, out i))
                {
                    stretchValue = i < 30 ? "30" : (i > 70 ? "70" : stretchValue);
                }

                // Get the file information from the selected file
                FileInfo fileInfo = new FileInfo(readPath);

                // Set up a file stream and binary reader for the selected file  
                fileStream = new FileStream(readPath, FileMode.Open, FileAccess.Read);
                br = new BinaryReader(fileStream);

                // Convert the file to a byte array  
                byteArray = br.ReadBytes((int)fileInfo.Length);

                // Close objects
                br.Close();
                fileStream.Close();
                fileStream.Dispose();

                // Create Request 		
                request = WebRequest.Create(serviceUri);
                request.Headers.Add("FileName", Path.GetFileName(readPath));
                request.Headers.Add("StretchValue", stretchValue);
                request.Method = "POST";
                request.ContentType = "application/octet-stream";
                request.ContentLength = byteArray.Length;
                request.Timeout = Constants.Timeout;

                // Write to Stream
                Stream requestStream = request.GetRequestStream();
                requestStream.Write(byteArray, 0, byteArray.Length);

                // Close the Stream object.
                requestStream.Close();

                // Get Response
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream dataStream = response.GetResponseStream();
                StreamReader reader = new StreamReader(dataStream);
                string responseFromServer = reader.ReadToEnd();
                responseFromServer = responseFromServer.Trim(new char[]{ '"' });

                // Save image to file
                byteArray = Convert.FromBase64String(responseFromServer);
                using (MemoryStream ms = new MemoryStream(byteArray))
                {
                    ms.Seek(0, SeekOrigin.Begin);
                    using (FileStream fs = new FileStream(writePath, FileMode.Create))
                    {
                        ms.CopyTo(fs);
                        fs.Flush();
                    }
                }

                // Release resources
                reader.Close();
                dataStream.Close();
                response.Close();
            }
            catch (WebException ex)
            {
                // When thrown by a WebRequest.Timeout, swallow if less than 5 time outs have occured
                if(!string.IsNullOrEmpty(ex.Message) && ex.Message.Contains("operation has timed out") && TimeoutCounter < 5)
                {
                    Console.WriteLine(Constants.ErrorOccured, ex.Message);
                    TimeoutCounter++;
                }
                else
                {
                    throw ex;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }
}
"@

Add-Type -TypeDefinition $code -Language CSharp	
iex "[SelfieMaskScript.Parser$id]::Main()"
